;;; Cube --- Cube library of FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2021 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://gitlab.com/FG42/FG42
;; Version: 3.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;; Cubes are the building blocks of any `FG42' editor.  Each `cube' is a
;; unit which defines different abilities in a deterministic and idempotent
;; way.  Cubes are composable and a composition of cubes creates an editor.
;;
;;; Code:
(require 'seq)
(require 'fg42/utils)


(defmacro defcube (cube-name props &rest body)
  "Define a cube with the given CUBE-NAME, a list of PROPS and a BODY."
  (let ((params-var (intern (format "%s-params" cube-name)))
        (active-var (intern (format "%s-active-p" cube-name))))
    `(progn
       (defvar ,params-var nil
         ,(format "Parameters for the '%s' cube." cube-name))
       (defun ,cube-name (&rest params)
         (when (not (boundp (quote ,active-var)))
           (progn
             (setq ,active-var t)
             (setq ,params-var params)
             ,@body))))))


(provide 'fg42/cube)
;;; cube.el ends here
